////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  ASHTTPConnection.h
//
//  Copyright 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  04/30/12  K.L.S.  Modified http code for updated API.
//	03/04/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CrossPlatform.h"
#include "ASFoundation/ASObject.h"

namespace ASFoundation
{
	class EXTERNAL_API ASHTTPConnection : public ASObject
	{
	public:
		static ASHTTPConnection* Create(void);
		static CPString PostHTTPMethod(void);
		static CPString PutHTTPMethod(void);
		static CPString GetHTTPMethod(void);
		static CPString DeleteHTTPMethod(void);
		static CPString UserAgentHTTPHeader(void);

		virtual ~ASHTTPConnection(void);

		bool IsSecured(void) const;
		void SetIsSecured(bool IsSecured);

		virtual unsigned long TimeoutInSeconds(void) const;
		virtual void SetTimeoutInSeconds(unsigned long TimeoutIntervalInSeconds);

		CPString HTTPMethod(void) const;
		void SetHTTPMethod(const CPString& HttpMethod);

		CPString URL(void) const;
		void SetURL(const CPString& Url);
		void AddValueForURLParameter(const CPString& ParameterValue, const CPString& URLParameter);
		CPDictionary URLParameters(void) const;
		void AddValueForHTTPHeader(const CPString& HeaderValue, const CPString& HeaderField);
		CPDictionary HTTPHeaders(void) const;
		CPData Body(void) const;
		void SetBody(const CPData& Body);

		//TODO: KLS - look to remove this functionality
		bool ShouldEncodeURL(void) const;
		void SetShouldEncodeURL(bool ShouldEncodeURL);

		virtual CPData SendSynchronousRequest(void) = 0;
		virtual CPData SendAsynchronousRequest(void) = 0;

	protected:
		CPString HTTPHeadersAsString(void) const;
		CPString URLParametersAsString(void) const;

		unsigned long m_TimeoutIntervalInSeconds;
		bool m_IsSecured;
		//  Pull this out later - client should do encoding - time constraint now
		bool m_ShouldEncodeURL;
		CPString m_HttpMethod;
		CPString m_Url;
		CPData m_Body;
		CPMutableDictionary m_URLParameters;
		CPMutableDictionary m_HTTPHeaders;

		ASHTTPConnection(void);
	};
}
