////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPAutoreleasePool.h
//    Cross-platform auto release pool class.
//
//  Copyright © 2011-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Reworked header includes.
//	11/25/11  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPId.h"

#if __OBJC__
#define AUTORELEASE_POOL_BLOCK @autoreleasepool
#else
#define AUTORELEASE_POOL_BLOCK
#endif

/// @brief   Auto Release Pool class
/// @details An auto release pool class that will automatically release the variable that were placed in the pool after it was created.
///
class CP_EXTERNAL_API CPAutoreleasePool
{
public:
	
	/// @brief     Default Constructor.
	/// @details   Creates an empty pool.
	CPAutoreleasePool(void);

	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPAutoreleasePool(void);
	
	///	@brief     Create a pool.
	///	@details   Create a pool used for auto releasing variable put into it.
	void CreatePool(void);

	///	@brief     Releases a pool.
	///	@details   Removes any references it has to shared resources and deletes the variables stored after it.
	void ReleasePool(void);

private:
	CPId m_Pool;
};
