////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPBase64Encoder.h
//    Cross-platform Base-64 encoding/decoding class.
//
//  Copyright © 2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	10/10/10  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPData.h"

/// @brief   Encoding/decoding class for data
/// @details This class is a helper class used for encoding or decoding data into a base 64 encoding.
///
class CP_EXTERNAL_API CPBase64Encoder
{
public:
	/// @brief     Encoding data into a base64 encoding.
	/// @details   This static method creates and returns a data object using base64 encoding.
	///	@param     DataToEncode  An underlying object to create the encoded data from.
	///	@return    This returns an instance of an encoded data object.
	static CPData EncodeData(const CPData& DataToEncode);

	/// @brief     Decoding data from a base64 encoding.
	/// @details   This static method creates and returns a data object after decoding base64 encoding.
	///	@param     DataToDecode  An underlying object to decode data from.
	///	@return    This returns an instance of a decoded data object.
	static CPData DecodeData(const CPData& DataToDecode);

private:
	CPBase64Encoder(void);
	~CPBase64Encoder(void);
};
