////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPCrossPlatformErrors.h
//    Cross-platform error codes for errors within CrossPlatform.
//
//  Copyright © 2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  05/12/13  K.L.S.  Created file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/LanguageSpecificMacros.h"

namespace CrossPlatformError
{
	CPString CrossPlatformErrorDomain(void);

	enum ErrorCode
	{
//		OSSpecificErrorId = -1,
//		OSSpecificExceptionId = -2
//
//
//
		// File system and file I/O related errors, with CPFilePathErrorKey or CPURLErrorKey containing path or URL
		CPFileNoSuchFileError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileNoSuchFileError, 4),                // Attempt to do a file system operation on a non-existent file
		CPFileLockingError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileLockingError, 255),                    // Couldn't get a lock on file
		CPFileReadUnknownError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileReadUnknownError, 256),            // Read error (reason unknown)
		CPFileReadNoPermissionError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileReadNoPermissionError, 257),  // Read error (permission problem)
		CPFileReadInvalidFileNameError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileReadInvalidFileNameError, 258), // Read error (invalid file name)
		CPFileReadCorruptFileError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileReadCorruptFileError, 259),         // Read error (file corrupt, bad format, etc)
		CPFileReadNoSuchFileError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileReadNoSuchFileError, 260),         // Read error (no such file)
		CPFileReadInapplicableStringEncodingError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileReadInapplicableStringEncodingError, 261), // Read error (string encoding not applicable) also NSStringEncodingErrorKey
		CPFileReadUnsupportedSchemeError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileReadUnsupportedSchemeError, 262), // Read error (unsupported URL scheme)
		CPFileReadTooLargeError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileReadTooLargeError, 263),               // Read error (file too large)
		CPFileReadUnknownStringEncodingError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileReadUnknownStringEncodingError, 264), // Read error (string encoding of file contents could not be determined)
		CPFileWriteUnknownError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileWriteUnknownError, 512),               // Write error (reason unknown)
		CPFileWriteNoPermissionError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileWriteNoPermissionError, 513),     // Write error (permission problem)
		CPFileWriteInvalidFileNameError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileWriteInvalidFileNameError, 514),// Write error (invalid file name)
		CPFileWriteFileExistsError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileWriteFileExistsError, 516),          // Write error (file exists)
		CPFileWriteInapplicableStringEncodingError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileWriteInapplicableStringEncodingError, 517),// Write error (string encoding not applicable) also NSStringEncodingErrorKey
		CPFileWriteUnsupportedSchemeError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileWriteUnsupportedSchemeError, 518), // Write error (unsupported URL scheme)
		CPFileWriteOutOfSpaceError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileWriteOutOfSpaceError, 540),          // Write error (out of disk space)
		CPFileWriteVolumeReadOnlyError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileWriteVolumeReadOnlyError, 642),  // Write error (readonly volume)

		// Other errors
		CPKeyValueValidationError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSKeyValueValidationError, 1024),           // KVC validation error
		CPFormattingError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFormattingError, 2048),                           // Formatting error
		CPUserCancelledError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSUserCancelledError, 3072),                     // User cancelled operation (this one often doesn't deserve a panel and might be a good one to special case)
		CPFeatureUnsupportedError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFeatureUnsupportedError, 3328),           // Feature unsupported error

		// Executable loading errors
		CPExecutableNotLoadableError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSExecutableNotLoadableError, 3584),     // Executable is of a type that is not loadable in the current process
		CPExecutableArchitectureMismatchError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSExecutableArchitectureMismatchError, 3585), // Executable does not provide an architecture compatible with the current process
		CPExecutableRuntimeMismatchError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSExecutableRuntimeMismatchError, 3586),  // Executable has Objective C runtime information incompatible with the current process
		CPExecutableLoadError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSExecutableLoadError, 3587),                   // Executable cannot be loaded for some other reason, such as a problem with a library it depends on
		CPExecutableLinkError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSExecutableLinkError, 3588),                   // Executable fails due to linking issues

		// Inclusive error range definitions, for checking future error codes
		CPFileErrorMinimum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileErrorMinimum, 0),
		CPFileErrorMaximum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFileErrorMaximum, 1023),

		CPValidationErrorMinimum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSValidationErrorMinimum, 1024),
		CPValidationErrorMaximum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSValidationErrorMaximum, 2047),

		CPExecutableErrorMinimum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSExecutableErrorMinimum, 3584),
		CPExecutableErrorMaximum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSExecutableErrorMaximum, 3839),

		CPFormattingErrorMinimum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFormattingErrorMinimum, 2048),
		CPFormattingErrorMaximum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSFormattingErrorMaximum, 2559),

		CPPropertyListReadCorruptError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSPropertyListReadCorruptError, 3840), // Error parsing a property list
		CPPropertyListReadUnknownVersionError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSPropertyListReadUnknownVersionError, 3841), // The version number in the property list is unknown
		CPPropertyListReadStreamError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSPropertyListReadStreamError, 3842), // Stream error reading a property list
		CPPropertyListWriteStreamError = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSPropertyListWriteStreamError, 3851), // Stream error writing a property list

		CPPropertyListErrorMinimum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSPropertyListErrorMinimum, 3840),
		CPPropertyListErrorMaximum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSPropertyListErrorMaximum, 4095),

		CPXPCConnectionInterrupted = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSXPCConnectionInterrupted, 4097),
		CPXPCConnectionInvalid = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSXPCConnectionInvalid, 4099),
		CPXPCConnectionReplyInvalid = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSXPCConnectionReplyInvalid, 4101),

		CPXPCConnectionErrorMinimum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSXPCConnectionErrorMinimum, 4096),
		CPXPCConnectionErrorMaximum = LANGUAGE_SPECIFIC_OBJECTIVE_C_OR_C(NSXPCConnectionErrorMaximum, 4224),

		// Cross-platform specific
		CPMemoryExceptionError = 20000,
		CPFileEndOfFileError = 20001,
		CPCrossPlatformErrorMinimum = 20000,
		CPCrossPlatformErrorMMaximum = 21000
	};
}
