////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPDebugLog.h
//    Routines used for outputting information to the console.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//	7/25/08  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Copyright © 2008 MYOB Technology Pty. Ltd., all rights reserved.
//
//  Change History (most recent first):
//
//	7/25/08  K.L.S.  Created File.
//
//  Notes:
//    Define _DEBUG or DEBUG in project setting debug build.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#if ! defined(NDEBUG) && ! defined(_DEBUG) && ! defined(DEBUG)
#warning "when including DebugLog.h you should define NDEBUG or _DEBUG. Defaulting to defining NDEBUG.
#define NDEBUG
#endif

#include "CrossPlatform/CPString.h"

void CP_EXTERNAL_API CPDebugLog(CPString Format, ...);
void CP_EXTERNAL_API CPNoDebugLog(CPString Format, ...);

/// @brief     Output a message to the console log.
/// @details   Will output a message to the console log if running a debug build, will do nothing in a release build. Only works in debug
///            build when DEBUG or _DEBUG is defined.
/// @param     Format message to be sent to the console log. It will perform parameter replacement on string.
/// @param     ... the replacement parameters.
#if defined(_DEBUG) || defined(DEBUG)
#define DebugLog CPDebugLog
#else
#define DebugLog(s, ...)
#endif
/// @brief     Outputs a message to the console with info about where in the code the message happened.
/// @details   Will output a message to the console log if running a debug build, will do nothing in a release build. Info includes routine
///            name, file name, and line number. Only works in debug build when DEBUG or _DEBUG is defined.
/// @param     s format message to be sent to the console log. It will perform parameter replacement on string.
/// @param     ... the replacement parameters.
#ifdef _MAC
#define TRACE_LOG(s, ...) DebugLog((s "    Function:%s File:%@ Line:%d"), ## __VA_ARGS__, __func__,  [@__FILE__ lastPathComponent], __LINE__);
#else
#define TRACE_LOG(s, ...) DebugLog((s "    Function:%s File:%@ Line:%d"), ## __VA_ARGS__, __FUNCTION__,  CPString(__FILE__).LastPathComponent().Ref(), __LINE__);
#endif
