////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPException.h
//    Class used to encapsulate a cross-platform exception.
//
//  Copyright © 2011-2013 Acclivity Group LLC., all rights reserved.
//
//  Change History (most recent first):
//
//  03/15/13  G.R.W.  Setup exceptions for Windows into some functions of CPFileHandle and added try-catch in calling code.
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//  11/30/11  K.L.S  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPDictionary.h"
#include "CrossPlatform/CPError.h"
#include "CrossPlatform/CPExceptionName.h"

#if _WINDOWS
class CException;
#endif

class CP_EXTERNAL_API CPException : public CPObject
{
public:
	static CPException ExceptionWithNameAndError(const CPString& Name, const CPError& Error);
	static CPException ExceptionWithNameAndReason(const CPString& Name, const CPString& Reason);
	static CPException ExceptionWithNameReasonAndUserInfo(const CPString& Name, const CPString& Reason, const CPDictionary& UserInfo);

	///	@brief     A cla3ss type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	CPException(void);
	CPException(const CPException& Object);
	~CPException(void);
	
	const CPException& operator=(const CPException& Object);

	void InitWithNameAndReason(const CPString& Name, const CPString& Reason);
	void InitWithNameReasonAndUserInfo(const CPString& Name, const CPString& Reason, const CPDictionary& UserInfo);
	void InitWithNameAndError(const CPString& Name, const CPError& Error);
	
	CPString Name(void) const;
	CPString Reason(void) const;
	CPDictionary UserInfo(void) const;

#if _WINDOWS
	static CPException ExceptionWithException(CException* Exception);
#endif
	
#if __OBJC__
	// Define methods that will be needed only when bridging to Cocoa
	static CPException ExceptionWithException(NSException* Exception);

	CPException(NSException* Exception);
	
	const CPException& operator=(NSException* Exception);
	
	operator NSException*() const;
#endif

};
