////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPPoint.h
//    A 2-dimensional point used in the CrossPlatform function calls.
//
//  Copyright © 2011 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//	11/22/11  K.L.S.  Created file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPCoordinate.h"

class CPSize;

class CP_EXTERNAL_API CPPoint
{
public:
	static CPPoint PointWithXYCoordinates(CPCoordinate X, CPCoordinate Y);

	CPPoint(void);
	CPPoint(const CPPoint& Point);
	virtual ~CPPoint(void);
	const CPPoint& operator=(const CPPoint& Point);

	bool operator==(const CPPoint& CompareWithPoint) const;	

	CPCoordinate XCoordinate(void) const;
	void SetXCoordinate(const CPCoordinate& XCoordinateIn);
	CPCoordinate YCoordinate(void) const;
	void SetYCoordinate(const CPCoordinate& YCoordinateIn);
	
	void OffsetBySize(const CPSize& OffsetSize);
	void OffsetByXYCoordinates(const CPCoordinate& XOffset, const CPCoordinate& YOffset);
	void OffsetByPoint(const CPPoint& OffsetPoint);

#ifdef __OBJC__
	CPPoint(const NSPoint& Point);
	const CPPoint& operator=(const NSPoint& Point);
	operator NSPoint(void) const;
#endif

protected:
	CPPoint(CPCoordinate X, CPCoordinate Y);
	
private:
	CPCoordinate m_X;
	CPCoordinate m_Y;
};
