////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPRect.h
//    A typedef file for types used in the CrossPlatform function calls.
//
//  Copyright © 2011-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Reworked header includes.
//	11/22/11  K.L.S.  Created file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPCoordinate.h"
#include "CrossPlatform/CPPoint.h"
#include "CrossPlatform/CPSize.h"

class CP_EXTERNAL_API CPRect
{
public:
	static const CPRect& EmptyRect(void);
	static CPRect RectWithOriginAndSize(const CPPoint& Origin, const CPSize& SizeIn);
	static CPRect RectWithXYCoordinatesWidthAndHeight(CPCoordinate X, CPCoordinate Y, CPCoordinate Width, CPCoordinate Height);
	
	CPRect(void);
	CPRect(const CPPoint& Origin, const CPSize& SizeIn);
	CPRect(const CPRect& RectIn);
	virtual ~CPRect(void);
	const CPRect& operator=(const CPRect& RectIn);

	const CPPoint& Origin(void) const;
	CPPoint& Origin(void);
	void SetOrigin(const CPPoint& OriginIn);
	const CPSize& Size(void) const;
	CPSize& Size(void);
	void SetSize(const CPSize& SizeIn);
	void SetWithXYCoordinatesWidthAndHeight(CPCoordinate X, CPCoordinate Y, CPCoordinate Width, CPCoordinate Height);
	CPCoordinate Width(void) const;
	void SetWidth(CPCoordinate WidthIn);
	CPCoordinate Height(void) const;
	void SetHeight(CPCoordinate HeightIn);
	
	void OffsetBySize(const CPSize& OffsetSize);
	void OffsetByXYCoordinates(const CPCoordinate& XOffset, const CPCoordinate& YOffset);
	void OffsetByPoint(const CPPoint& OffsetPoint);
	void IncreaseSizeBySize(CPSize Size);
	void IncreaseSizeByWidthAndHeight(CPCoordinate WidthIn, CPCoordinate HeightIn);
	void ScaleSizeBySizeScaleRatio(CPSize SizeScaleFactor);
	void ScaleSizeByScaleFactor(CPCoordinate ScaleFactor);
	void ScaleSizeByWidthAndHeightScaleRatio(CPCoordinate WidthScaleFactor, CPCoordinate HeightScaleFactor);

	CPCoordinate MaxX(void) const;
	CPCoordinate MaxY(void) const;
	CPCoordinate MidX(void) const;
	CPCoordinate MidY(void) const;
	CPCoordinate MinX(void) const;
	CPCoordinate MinY(void) const;

	bool operator==(const CPRect& Rect) const;
	bool IsEqualToRect(const CPRect& Rect) const;
	bool IsEmpty(void) const;

	CPRect InsetRect(CPCoordinate DX, CPCoordinate DY) const;
	CPRect OffsetRect(CPCoordinate DX, CPCoordinate DY) const;
	CPRect IntegralRect(void) const;
	CPRect UnionRect(const CPRect& Rect) const;
	CPRect IntersectionRect(const CPRect& Rect) const;
	//	CPRect DivideRect(CPRect* slice, CPRect* rem, CPCoordinate amount, CPRectEdge edge) const;
	bool PointInRect(const CPPoint& Point) const;
	bool ContainsRect(const CPRect& Rect) const;
	bool IntersectsRect(const CPRect& Rect) const;

#ifdef __OBJC__
	CPRect(const NSRect& Rect);
	const CPRect& operator=(const NSRect& Rect);
	operator NSRect(void) const;
#endif

protected:
	CPRect(CPCoordinate X, CPCoordinate Y, CPCoordinate Width, CPCoordinate Height);

private:
	CPPoint m_Origin;
	CPSize m_Size;
};
