////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPSize.h
//    A 2-dimensional size used in the CrossPlatform function calls.
//
//  Copyright © 2011 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//	11/22/11  K.L.S.  Created file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CPOS_API.h"
#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/LanguageSpecificMacros.h"
#include "CrossPlatform/CPCoordinate.h"

class CP_EXTERNAL_API CPSize
{
public:
	static CPSize SizeWithWidthAndHeight(CPCoordinate Width, CPCoordinate Height);
	static CPSize EmptySize(void);

	CPSize(void);
	CPSize(const CPSize& Size);
	virtual ~CPSize(void);
	const CPSize& operator=(CPSize Size);

	bool operator==(const CPSize& CompareWithSize) const;

	CPCoordinate Width(void) const;
	void SetWidth(CPCoordinate WidthIn);
	CPCoordinate Height(void) const;
	void SetHeight(CPCoordinate HeightIn);

	void IncreaseSizeBySize(CPSize Size);
	void IncreaseSizeByWidthAndHeight(CPCoordinate WidthIncrease, CPCoordinate HeightIncrease);

	void ScaleSizeBySizeScaleRatio(CPSize SizeScaleFactor);
	void ScaleSizeByScaleFactor(CPCoordinate ScaleFactor);
	void ScaleSizeByWidthAndHeightScaleRatio(CPCoordinate WidthScaleFactor, CPCoordinate HeightScaleFactor);

	bool IsEmpty(void) const;
	void Empty(void);

#ifdef __OBJC__
	CPSize(const NSSize& Size);
	const CPSize& operator=(const NSSize& Size);
	operator NSSize(void) const;
#endif

protected:
	CPSize(CPCoordinate Width, CPCoordinate Height);

private:
	CPCoordinate m_Width;		// should never be negative
	CPCoordinate m_Height;	// should never be negative
};
