////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPTimer.h
//    Class used to create timer events. Timer events are events that you need to fire at some future point in time.  
//
//  Copyright © 2013 Acclivity Group LLC, all rights reserved.
//
//  02/28/13  K.L.S.  Added file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/// @brief    CPTimer objects represent a class for handling elapsed timing of events.
/// @details  You use the CPTimer class to create timer objects or, more simply, timers. A timer waits until a certain time interval has
///           elapsed and then fires, sending a specified message to a target object. For example, you could create an CPTimer object that
///           sends a message to a window, telling it to update itself after a certain time interval.
///
///           Timers work in conjunction with run loops. To use a timer effectively, you should be aware of how run loops operate—
///           For MacOS see NSRunLoop and Threading Programming Guide. Note in particular that run loops retain their timers, so you can
///           release a timer after you have added it to a run loop.
///
///           A timer is not a real-time mechanism; it fires only when one of the run loop modes to which the timer has been added is running
///           and able to check if the timer’s firing time has passed. Because of the various input sources a typical run loop manages, the
///           effective resolution of the time interval for a timer is limited to on the order of 50-100 milliseconds. If a timer’s firing
///           time occurs during a long callout or while the run loop is in a mode that is not monitoring the timer, the timer does not fire
///           until the next time the run loop checks the timer. Therefore, the actual time at which the timer fires potentially can be a
///           significant period of time after the scheduled firing time.
///
///           CPTimer uses an NSTimer which is “toll-free bridged” with its Core Foundation counterpart, CFRunLoopTimerRef. See “Toll-Free
///           Bridging” for more information on toll-free bridging.
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSTimer_Class
///

#pragma once

#include "CrossPlatform/CPObject.h"

class CP_EXTERNAL_API CPTimer
{
public:
	/// @brief  Default constructor.
	CPTimer(void);

	/// @brief  destructor.
	virtual ~CPTimer();
	
private:
};
