////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPUInteger.h
//    Cross-platform unsigned integer class.
//
//  Copyright © 2010-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	09/18/10  K.L.S.  Created File.
//
//  Notes:
//    When building 32-bit applications, NSUInteger is a 32-bit unsigned integer. A 64-bit application treats NSUInteger as a 64-bit
//    unsigned integer.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"

#if __LP64__
typedef unsigned long CPUInteger;
#elif defined _WIN64
typedef unsigned long long CPUInteger;
#else
typedef unsigned int CPUInteger;
#endif

extern const CPUInteger CP_EXTERNAL_API CPNotFound;
extern const CPUInteger CP_EXTERNAL_API CPUIntegerMin;
extern const CPUInteger CP_EXTERNAL_API CPUIntegerMax;

namespace CPUIntegerUtilites
{
	CPUInteger CP_EXTERNAL_API Min(CPUInteger AValue, CPUInteger BValue);
	CPUInteger CP_EXTERNAL_API Min(int AValue, CPUInteger BValue);
	CPUInteger CP_EXTERNAL_API Min(CPUInteger AValue, int BValue);
	CPUInteger CP_EXTERNAL_API Max(CPUInteger AValue, CPUInteger BValue);
	CPUInteger CP_EXTERNAL_API Max(int AValue, CPUInteger BValue);
	CPUInteger CP_EXTERNAL_API Max(CPUInteger AValue, int BValue);
};
