////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPUserDefaults.h
//    Cross platform user defaults class.
//
//  Copyright © 2010-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	06/08/10  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPInteger.h"
#include "CrossPlatform/CPFloat.h"
#include "CrossPlatform/CPDouble.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPArray.h"
#include "CrossPlatform/CPDate.h"
#include "CrossPlatform/CPData.h"
#include "CrossPlatform/CPDictionary.h"
#include "CrossPlatform/CPURL.h"

class CPDictionary;

class CP_EXTERNAL_API CPUserDefaults : public CPObject
{
public:
	static CPUserDefaults StandardUserDefaults(void);
	static CPUserDefaults StandardAllUsersDefaults(void);
	static CPUserDefaults StandardUserDefaultsWithBundleIdentifier(const CPString& BundleIdentifier);
	static CPUserDefaults StandardAllUsersDefaultsWithBundleIdentifier(const CPString& BundleIdentifier);

	virtual ~CPUserDefaults(void);
	
	void RegisterDefaults(const CPDictionary& DefaultsInitializationDictionary) const;
	void RegisterDefaultForKey(const CPObject& DefaultValue, const CPString& DefaultsKey) const;

	CPArray ArrayForKey(const CPString& DefaultsKey) const;
	CPBool BoolForKey(const CPString& DefaultsKey) const;
	CPData DataForKey(const CPString& DefaultsKey) const;
	CPDictionary DictionaryForKey(const CPString& DefaultsKey) const;
	CPNumber NumberForKey(const CPString& DefaultsKey) const;
	CPFloat FloatForKey(const CPString& DefaultsKey) const;
	CPInteger IntegerForKey(const CPString& DefaultsKey) const;
	CPString StringForKey(const CPString& DefaultsKey) const;
	CPDouble DoubleForKey(const CPString& DefaultsKey) const;
	CPObject ObjectForKey(const CPString& DefaultsKey) const;

	void SetArrayForKey(const CPArray& DefaultsValue, const CPString& DefaultsKey);
	void SetBoolForKey(const CPBool& DefaultsValue, const CPString& DefaultsKey);
	void SetDataForKey(const CPData& DefaultsValue, const CPString& DefaultsKey);
	void SetDictionaryForKey(const CPDictionary& DefaultsValue, const CPString& DefaultsKey);
	void SetFloatForKey(const CPFloat& DefaultsValue, const CPString& DefaultsKey);
	void SetIntegerForKey(const CPInteger& DefaultsValue, const CPString& DefaultsKey);
	void SetStringForKey(const CPString& DefaultsValue, const CPString& DefaultsKey);
	void SetDoubleForKey(const CPDouble& DefaultsValue, const CPString& DefaultsKey);
	void SetObjectForKey(const CPObject& DefaultsValue, const CPString& DefaultsKey);

	bool RemoveObjectForKey(const CPString& DefaultsKey) const;

	bool RemoveUserDefaults(void) const;

	bool AccessingAllUsersDefaults(void) const;
	bool Synchronize(void) const;

protected:
	CPUserDefaults(void);
	CPUserDefaults(bool AccessAllUsersDefaults);
	CPUserDefaults(const CPString& BundleIdentifier, bool AccessAllUsersDefaults);

private:
	CPString m_BundleIdentifier;
	bool m_AccessAllUsersDefaults;
};
