////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  OBImage.h
//      Definition of OBImage class. Class is cross-platform handling an image. This is for class for use with obsolete drawing.
//
//  Copyright: (C) 2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first): 
//
//  10/10/10  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/PlatformSpecificMacros.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPData.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPSize.h"
#include "CrossPlatform/CPRect.h"
#ifdef _WINDOWS
#include <Windows.h>
#include <GDIplus.h>
#endif

class EXTERNAL_API OBImage : public CPObject
{
public:
	static OBImage ImageWithContentsOfFile(const CPString& Path);
	static OBImage ImageWithData(const CPData& ImageData);

	OBImage(void);
	virtual ~OBImage(void);

	OBImage(const OBImage& ImageIn);
	const OBImage& operator=(const OBImage& ImageIn);
	
	void Release(void);

	/// @brief     Checks to see if the object is empty.
	///
	/// @details   This checks to see if the object is empty and returns its status.
	///
	/// @return    true if the object was empty, otherwise false.
	///
	virtual bool IsEmpty(void) const;

	/// @brief     Removes references to any resources. Makes the underlying pointer NULL.
	///
	/// @details   This will release any refernences and will deallocate any resources no longer in use. The object will not be using the
	///            empty object but will be set to NULL.
	///
	virtual void Nullify(void);

	CPData Data(void) const;
	void InitWithImageData(const CPData& ImageData);
	void InitWithContentsOfFile(const CPString& Path);
	
	CPSize Size(void) const;

#ifdef __OBJC__
	OBImage(NSImage* ImageIn);
	const OBImage& operator=(NSImage* ImageIn);
	CPData m_AlternateData;
#else
	OBImage(const Gdiplus::Image* ImageIn);
	const OBImage& operator=(const Gdiplus::Image* ImageIn);
	operator Gdiplus::Image*(void) const;
#endif
};
